# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of canonical tight windows calculation

Ported from ltfat_2.1.0/gabor/gabtight.m

.. moduleauthor:: Denis Arrivault,
                  Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.gabor.dgtlength import dgtlength
from ltfatpy.sigproc.fir2long import fir2long
from ltfatpy.sigproc.long2fir import long2fir
from ltfatpy.gabor.gabframediag import gabframediag
from ltfatpy.comp.comp_gabtight_long import comp_gabtight_long


def gabtight(g, a, M, L=None):
    """Canonical tight window of Gabor frame

    - Usage:

        | ``gt = gabtight(None, a, M, L)``
        | ``gt = gabtight(g, a, M)``
        | ``gt = gabtight(g, a, M, L)``

    - Input parameters:

    :param g: Gabor window
    :type g: numpy.ndarray or str or dict
    :param int a: Length of time shift
    :param int M: Number of modulations
    :param int L: Length of window (optional except if **g** is None)

    - Output parameters:

    :return: Canonical tight window
    :rtype: numpy.ndarray

    ``gabtight(None, a, M, L)`` computes a nice tight window of length **L**
    for a lattice with parameters **a**, **M**. The window is not an FIR
    window, meaning that it will only generate a tight system if the system
    length is equal to **L**.

    ``gabtight(g, a, M)`` computes the canonical tight window of the Gabor
    frame with window **g** and parameters **a**, **M**.

    The window **g** may be a vector of numerical values, a text string or a
    dictionary. See the help of :func:`~ltfatpy.gabor.gabwin` for more details.

    If the length of **g** is equal to **M**, then the input window is assumed
    to be a FIR window. In this case, the canonical dual window also has
    length of **M**. Otherwise the smallest possible transform length is
    chosen as the window length.

    ``gabtight(g, a, M, L)`` returns a window that is tight for a system of
    length **L**. Unless the input window **g** is a FIR window, the returned
    tight window will have length **L**.

    If ``a > M`` then an orthonormal window of the Gabor Riesz sequence
    with window **g** and parameters **a** and **M** will be calculated.

    - Examples:

        The following example shows the canonical tight window of the Gaussian
        window. This is calculated by default by
        :func:`~ltfatpy.gabor.gabtight` if no window is specified:

        >>> import matplotlib.pyplot as plt
        >>> from ltfatpy import gabtight
        >>> a = 20
        >>> M = 30
        >>> L = 300
        >>> gt = gabtight(None, a, M, L)
        >>> # Plot in the time-domain
        >>> _ = plt.plot(gt)
        >>> plt.show()

    .. image:: images/gabtight.png
       :width: 700px
       :alt: pgauss gabtight image
       :align: center

    .. seealso::  :func:`~ltfatpy.gabor.gabdual.gabdual`,
                  :func:`~ltfatpy.gabor.gabwin.gabwin`,
                  :func:`~ltfatpy.sigproc.fir2long.fir2long`,
                  :func:`~ltfatpy.gabor.dgt.dgt`
    """
    # Verify a, M and L
    if g is None:
        g = 'gauss'

    if L is None:
        if not isinstance(g, np.ndarray):
            Ls = 1
        else:
            Ls = g.shape[0]
        L = dgtlength(Ls, a, M)
    else:
        Luser = dgtlength(L, a, M)
        if L != Luser:
            raise ValueError(("Incorrect transform length L={0:d} specified." +
                              " Next valid length  is L={1:d}. See the help" +
                              " of DGTLENGTH for the requirements.").format(L,
                             Luser))

    # Determine the window
    (g, info) = _call_gabwin(g, a, M, L)

    if L < info['gl']:
        raise ValueError('Window is too long.\n')
    R = 1
    if g.ndim > 1:
        R = g.shape[1]

    # Are we in the Riesz sequence of in the frame case
    scale = 1
    if a > M*R:
        # Handle the Riesz basis (dual lattice) case.
        # Swap a and M, and scale differently.
        scale = np.sqrt(a/M)
        a, M = M, a

    # Compute the rectangular case
    if info['gl'] <= M and R == 1:
        # Diagonal of the frame operator
        d = gabframediag(g, a, M, L)
        gt = g / np.sqrt(long2fir(d, info['gl']))
    else:
        # Long window case
        # Just in case, otherwise the call is harmless.
        g = fir2long(g, L)
        gt = comp_gabtight_long(g, a, M) * scale

    # post process result
    if np.issubdtype(g.dtype, np.floating):
        # If g is real then the output is known to be real.
        gt = gt.real

    return gt


def _call_gabwin(g, a, M, L):
    # gabwin is imported in a different function to avoid circular imports
    from ltfatpy.gabor.gabwin import gabwin
    return gabwin(g, a, M, L)

if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the pherm function

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import unittest
import random
import numpy as np
from ltfatpy.fourier.pherm import pherm
from ltfatpy.tests.datasets.read_ref_mat import read_ref_mat
from ltfatpy.tests.datasets.get_dataset_path import get_dataset_path


class TestPherm(unittest.TestCase):
    """ Unittest class for Fourier pherm"""
    def setUp(self):
        self.L = random.randint(10, 100)
        self.filename = get_dataset_path('pherm_ref.mat')

    def tearDown(self):
        print('Test done')

    def test_default(self):
        (g_r1, _) = pherm(self.L, 1)
        (g_r2, _) = pherm(self.L, 1, phase='accurate', orthtype='noorth')
        mess = "\nTest default values with L = " + str(self.L) + "\n"
        self.assertTrue(np.array_equal(g_r1, g_r2), mess)

    def test_exceptions(self):
        mess = "\nException TypeError should be raised with declaration "
        mess += "pherm((10, 2), 1)\n"
        self.assertRaises(TypeError, pherm, (10, 2), 1, mess)
        mess = "\nException TypeError should be raised with declaration "
        mess += "pherm(10.2, 1)\n"
        self.assertRaises(TypeError, pherm, 10.2, 1, mess)
        mess = "\nException TypeError should be raised with declaration "
        mess += "pherm(10, 1, (1,1))\n"
        self.assertRaises(TypeError, pherm, 10, 1, (1, 1))
        mess = "\nException TypeError should be raised with declaration "
        mess += "pherm(10, 'foo')\n"
        self.assertRaises(TypeError, pherm, 10, 'foo')

    def test_calc(self):
        data = read_ref_mat(self.filename)
        for inputs, outputs in data:
            out = pherm(**inputs)
            msg = ('Wrong output in pherm with inputs ' +
                   str(inputs))
            if inputs['orthtype'] == 'qr':
                np.testing.assert_array_almost_equal(np.abs(out[0]),
                                                     np.abs(outputs[0]), 6,
                                                     msg)
                np.testing.assert_array_almost_equal(out[1], outputs[1], 6,
                                                     msg)
            else:
                np.testing.assert_array_almost_equal(out[0], outputs[0], 6,
                                                     msg)
                np.testing.assert_array_almost_equal(out[1], outputs[1], 6,
                                                     msg)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestPherm)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the s0norm function

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import unittest
import numpy as np
# from math import *
# import matplotlib.pyplot as plt
# from fractions import gcd

from ltfatpy.gabor.s0norm import s0norm
from ltfatpy.tests.datasets.read_s0norm_ex_mat import S0Examples
from ltfatpy.tests.datasets.get_dataset_path import get_dataset_path


class TestS0Norm(unittest.TestCase):
    # Called before the tests.
    def setUp(self):
        self.filename = get_dataset_path('s0_norm_test.mat')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_default(self):
        """ Comparing results with Matlab generated ones
        """
        pass
        dgs = S0Examples(self.filename)
        (gl, R, dim, rel, G, S0, S0Shape) = dgs.read_next_frame()
        while gl != '':
            dim = dim - 1
            mess = "\ngl = " + str(gl) + "\nR = " + str(R)
            mess += "\ndim = " + str(dim) + "\nrel = " + str(rel)
            mess += "\nG = " + str(G) + "\nS0 = " + str(S0)
            mess += "\nS0Shape = " + str(S0Shape)
            if rel == 'rel':
                s0r = s0norm(G, True, dim)
            else:
                s0r = s0norm(G, False, dim)
            mess += "\ns0r = " + str(s0r)
            self.assertTrue(np.linalg.norm(np.squeeze(s0r)-S0) <= 1e-10, mess)
            if(len(s0r.shape) > 1):
                self.assertEqual(s0r.shape, tuple(S0Shape), mess)
            (gl, R, dim, rel, G, S0, S0Shape) = dgs.read_next_frame()

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestS0Norm)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the long2fir function

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import unittest
import numpy as np

from ltfatpy.sigproc.long2fir import long2fir


class TestSigprocLong2fir(unittest.TestCase):
    """ Unittest class for Sigproc long2fir"""
    digit = 9

    def setUp(self):
        pass

    def tearDown(self):
        print('Test done')

    def test_default(self):
        self.assertRaises(TypeError, long2fir, "foo", 10)
        self.assertRaises(ValueError, long2fir, np.ones(10), None)
        self.assertRaises(ValueError, long2fir, np.ones(10), 15)
        self.assertRaises(ValueError, long2fir, np.ones(10), 5,
                          centering='foo')

        gin = np.arange(20, dtype='f4')

        g = long2fir(gin, 10)
        np.testing.assert_array_equal(g, np.hstack((np.arange(5),
                                                    np.arange(15, 20))))
        g = long2fir(gin, 11)
        np.testing.assert_array_equal(g, np.hstack((np.arange(6),
                                                    np.arange(15, 20))))
        g = long2fir(gin, 10, centering='wp')
        np.testing.assert_array_equal(g, np.hstack((np.arange(5),
                                                    np.array([0]),
                                                    np.arange(16, 20))))
        g = long2fir(gin, 11, centering='hp')
        np.testing.assert_array_equal(g, np.hstack((np.arange(5),
                                                    np.array([0]),
                                                    np.arange(15, 20))))

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestSigprocLong2fir)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of FFT for real valued data computation

Ported from ltfat_2.1.0/fourier/fftreal.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np


def fftreal(f, N=None, dim=0):
    """FFT for real valued input data

    - Usage:

        | ``c = fftreal(f)``
        | ``c = fftreal(f, N, dim)``

    - Input parameters:

    :param numpy.ndarray f: Real valued input array
    :param int N: FFT length
    :param int dim: Axis over which to compute the FFT. By default the first
        axis is used.

    - Output parameters:

    :returns: Discrete Fourier coefficients of **f** for positive frequencies
    :rtype: numpy.ndarray

    ``fftreal(f)`` computes the coefficients corresponding to the positive
    frequencies of the FFT of the real valued input signal **f**.

    The function takes the same arguments as :func:`numpy.fft.rfft`. See the
    help on this function for a thorough description.

    .. note::

        This Python port doesn't use the C core of LTFAT to implement the
        FFT for real arrays as in the Octave version.

        Instead it direclty uses the implementation available in numpy.

        This can lead to sightly different results when comparing the results
        of the Octave and Python versions of fftreal.

    .. seealso:: :func:`~ltfatpy.fourier.ifftreal.ifftreal`,
        :func:`~ltfatpy.fourier.dft.dft`

    """

    # Note: There is a bug in fftreal in ltfat 2.1.0 for Octave. It has been
    # reported and confirmed here:
    # http://sourceforge.net/p/ltfat/bugs/118/
    # This Python port is not subject to this bug.

    return np.fft.rfft(f, N, dim)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of idgtreal calculation

Ported from ltfat_2.1.0/gabor/idgtreal.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.gabor.dgtlength import dgtlength
from ltfatpy.gabor.gabwin import gabwin
from ltfatpy.comp.comp_isepdgtreal import comp_isepdgtreal
from ltfatpy.tools.postpad import postpad
from ltfatpy.comp.comp_sigreshape_post import comp_sigreshape_post


def idgtreal(coef, g, a, M, Ls=None, pt='freqinv'):
    """Inverse discrete Gabor transform for real-valued signals

    - Usage:
        | ``(f, g) = idgtreal(c, g, a, M)``
        | ``(f, g) = idgtreal(c, g, a, M, Ls)``
        | ``(f, g) = idgtreal(c, g, a, M, Ls, pt)``

    - Input parameters:

    :param numpy.ndarray c: Array of coefficients
    :param g: Window function
    :param int a: Length of time shift
    :param int M: Number of channels
    :param int Ls: Length of signal
    :param str pt: 'freqinv' or 'timeinv'. Default is 'freqinv'.
    :type g: str, dict or numpy.ndarray

    - Output parameters:

    :returns: ``(f, g)``
    :rtype: tuple
    :var numpy.ndarray f: signal
    :var numpy.ndarray g: window


    ``idgtreal(c, g, a, M)`` computes the Gabor expansion of the input
    coefficients **c** with respect to the real-valued window **g**, time
    shift **a** and number of channels **M**. **c** is assumed to be the
    positive frequencies of the Gabor expansion of a real-valued signal.

    It must hold that ``c.shape[0] == np.floor(M/2)+1``. Note that since the
    correct number of channels cannot be deduced from the input, ``idgtreal``
    takes an additional parameter as opposed to
    :func:`~ltfatpy.gabor.idgt.idgt`.

    The window **g** may be a vector of numerical values, a text string or a
    dictionary. See the help of :func:`~ltfatpy.gabor.gabwin.gabwin` for more
    details.

    ``idgtreal(c, g, a, M, Ls)`` does as above but cuts or extends **f** to
    length **Ls**.

    ``(f, g) = idgtreal(...)`` outputs the window used in the transform. This
    is useful if the window was generated from a description in a string or
    dictionary.

    For perfect reconstruction, the window used must be a dual window of the
    one used to generate the coefficients.

    If **g** is a row vector, then the output will also be a row vector. If
    **c** is 3-dimensional, then ``idgtreal`` will return a matrix consisting
    of one column vector for each of the TF-planes in **c**.

    See the help on :func:`~ltfatpy.gabor.idgt.idgt` for the precise definition
    of the inverse Gabor transform.

    - Additional parameters

    ``idgtreal``  optionnaly takes a **pt** arguments that can take the
    following values:

        ==========  ===========================================================
        'freqinv'   Compute a ``idgtreal`` using a frequency-invariant phase.
                    This is the default convention described in the help for
                    :func:`~ltfatpy.gabor.dgt.dgt`.

        'timeinv'   Compute a ``idgtreal`` using a time-invariant phase. This
                    convention is typically used in filter bank algorithms.
        ==========  ===========================================================

    - Examples

        The following example demonstrates the basic principles for getting
        perfect reconstruction (short version)::

            >>> from ltfatpy import greasy
            >>> from ltfatpy import dgtreal
            >>> f = greasy()[0]   # Input test signal
            >>> a = 32  # time shift
            >>> M = 64  # frequency shift
            >>> gs = {'name': 'blackman', 'M': 128}  # synthesis window
            >>> # analysis window
            >>> ga = {'name' : ('dual', gs['name']), 'M' : gs['M']}
            >>> (c, Ls) = dgtreal(f, ga, a, M)[0:2]  # analysis
            >>> r = idgtreal(c, gs, a, M, Ls)[0]  # synthesis
            >>> np.linalg.norm(f-r) < 1e-10 # test
            True

    .. seealso::  :func:`~ltfatpy.gabor.idgt.idgt`,
        :func:`~ltfatpy.gabor.gabwin.gabwin`,
        :func:`~ltfatpy.gabor.gabdual.gabdual`, :func:`dwilt`
    """
    if (not isinstance(g, np.ndarray) and not isinstance(g, str) and
       not isinstance(g, dict)):
        raise TypeError('g must be a numpy.array or str or dict.')

    if (isinstance(g, np.ndarray) and g.size < 2):
        raise ValueError('g must be a vector (you probably forgot to supply' +
                         ' the window function as input parameter.)')

    # Define initial value for flags and key/value pairs.
    if coef.ndim < 2:
        raise ValueError('coef must have at least 2 dimensions')
    N = coef.shape[1]
    if coef.ndim > 2:
        W = coef.shape[2]
    else:
        W = 1

    # Make a dummy call to test the input parameters
    Lsmallest = dgtlength(1, a, M)
    M2 = np.floor(M/2)+1

    if M2 != coef.shape[0]:
        mess = ('Mismatch between the specified number of channels ' +
                'and the size of the input coefficients: ' +
                'M2 = {0:f}, coef.shape = {1:s}')
        raise ValueError(mess.format(M2, '%s' % (coef.shape, )))

    L = N * a

    if L % Lsmallest > 0:
        raise ValueError('Invalid size of coefficient array.')

    # Determine the window

    (g, info) = gabwin(g, a, M, L)

    if L < info['gl']:
        raise ValueError('Window is too long.')

    if not np.issubdtype(g.dtype, np.floating):
        raise ValueError('The window must be real-valued.')

    # verify pt
    if pt == 'timeinv':
        pt = 1
    elif pt == 'freqinv':
        pt = 0
    else:
        mes = "pt (" + str(pt) + ") argument should be 'timeinv' or 'freqinv'."
        raise ValueError(mes)

    # Do the actual computation.
    f = comp_isepdgtreal(coef, g, a, M, pt)

    # Cut or extend f to the correct length, if desired.
    if Ls is not None:
        f = postpad(f, Ls)
    else:
        Ls = L
    f = comp_sigreshape_post(f, Ls, 0, (0, W))
    return (f, g)

if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of original permuted shape restoration

Ported from ltfat_2.1.0/comp/assert_sigreshape_post.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np


def assert_sigreshape_post(f, dim, permutedshape, order):
    """Restore the original permuted shape

    - Input parameters:

    :param numpy.ndarray f: Reshaped signal as returned in **f** by
        :func:`~ltfatpy.comp.assert_sigreshape_pre.assert_sigreshape_pre`
    :param int dim: Verified dim as returned in **dim** by
        :func:`~ltfatpy.comp.assert_sigreshape_pre.assert_sigreshape_pre`
    :param tuple permutedshape: **permutedshape** value as returned by
        :func:`~ltfatpy.comp.assert_sigreshape_pre.assert_sigreshape_pre`
    :param tuple order: **order** value as returned by
        :func:`~ltfatpy.comp.assert_sigreshape_pre.assert_sigreshape_pre`

    - Output parameters:

    :returns: The signal reshaped to its original shape
    :rtype: numpy.ndarray

    .. warning::
        This function returns a view of **f** if possible. Any value changed
        in this view will also be changed in **f**.
    """

    f = f.reshape(permutedshape)

    if dim > 0:
        # Undo the permutation.
        f = f.transpose(np.argsort(order))

    return f



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module that extends fir windows with zeros

Ported from ltfat_2.1.0/sigproc/fir2long.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.fourier.middlepad import middlepad


def fir2long(gin, Llong):
    """Extend FIR window to LONG

    - Usage:

        | ``g= fir2long(g, Llong)``

    - Input parameters:

    :param numpy.ndarray g: input window
    :param int Llong: new length

    - Ouput parameters:

    :return: extended window
    :rtype: numpy.ndarray

    ``fir2long(g,Llong)`` will extend the FIR window **g** to a length
    **Llong** window by inserting zeros. Note that this is a slightly
    different behaviour than :py:meth:`~ltfatpy.fourier.middlepad`.

    ``fir2long`` can also be used to extend a FIR window to a longer FIR
    window, for instance in order to satisfy the usual requirement that the
    window length should be divisible by the number of channels.

    .. seealso::  :func:`~ltfatpy.sigproc.long2fir.long2fir`,
        :func:`~ltfatpy.fourier.middlepad.middlepad`
    """
    if not isinstance(gin, np.ndarray):
        raise TypeError("gin must be a numpy array")
#     if gin.ndim > 1:
#         raise ValueError("gin must be a vector (dim = 1)")
    Lfir = gin.shape[0]
    if Lfir > Llong:
        raise ValueError('Llong must be larger than length of window.')
    if Lfir % 2 == 0:
        # HPE middlepad works the same way as the FIR extension (e.g. just
        # inserting zeros) for even-length signals.
        gout = middlepad(gin, Llong, centering='hp')
    else:
        # WPE middlepad works the same way as the FIR extension (e.g. just
        # inserting zeros) for odd-length signals.
        gout = middlepad(gin, Llong)
    return gout



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Read .mat files generated with the Octave version of ltfat for validation

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np
import scipy.io


def _squeeze_array(val):
    """Remove single-dimensional entries from the shape of Numpy arrays

    - Input parameter:

    :param val: A Python object of any type

    - Output parameter:

    :returns: If **val** is an instance of numpy.ndarray, the squeezed version
        of **val** is returned, otherwise, **val** is returned
    :rtype: Same type as **val**

    If **val** is an instance of numpy.ndarray, this function returns the
    input array **val**, but with all of the dimensions of length 1 removed.
    The output is always **val** itself or a view into **val**.
    """
    res = val
    if isinstance(val, np.ndarray):
        res = np.squeeze(res)
    return res


def _adapt_type(val):
    """Adapt the type of the input

    - Input parameter:

    :param val: A Python object of any type

    - Output parameter:

    :returns: If the type and content of **val** can be adapted to better match
        the expected types and values used in the ltfatpy functions, the
        adapted version of **val** is returned, otherwise, **val** is returned

    The following conversions are done when **val** is an instance of
    numpy.ndarray:

    - Arrays with dtype numpy.unicode are converted to standard Python
      strings, and if their content is ``u'_bool_True_'`` or
      ``u'_bool_False_'`` they are converted to the corresponding boolean value
      (output of type bool)
    - If the array contains a single value, it is extracted from the array and
      converted to the appropriate Python object. If this value is a float
      having an integer value, it is converted to int.

    """
    res = val
    if isinstance(val, np.ndarray):
        if np.issubdtype(val.dtype, np.str_):
            # convert the unicode numpy arrays to standard unicode strings
            res = val.tolist()
            # convert boolean values
            if val == "_bool_True_":
                res = True
            elif val == "_bool_False_":
                res = False
        elif val.size == 1:
            # convert the single value in the array to the corresponding
            # standard Python object
            res = np.squeeze(val).tolist()
            # force type int for integer values
            if isinstance(res, float):
                if res.is_integer():
                    res = int(res)
    return res


def read_ref_mat(filename, squeeze_arrays=True, adapt_types=True):
    """Read reference data saved in a .mat file

    - Input parameter:

    :param str filename: File name of the .mat file containing reference data
        saved in the expected format using Octave or MATLAB
    :param bool squeeze_arrays: Flag specifying if the arrays must be squeezed
        when reading the data (if ``True``, the dimensions of length 1 are
        removed in the shapes of the arrays)
    :param bool adapt_types: Flag specifying if the data types must be adapted
        when reading the data (if ``True`` the types are adapted, see below for
        details)

    - Output parameters:

    :returns: A list of tuples of the form ``(inputs, outputs)``, each tuple
        giving the outputs expected for a given set of inputs
    :rtype: list
    :var dict inputs: The inputs as expected by the tested Python function
        (note that it implies that there can be some differences with the
        inputs used to generate the data in Octave or MATLAB)
    :var list outputs: The expected outputs when running the tested Python
        function with inputs given in **inputs**

    In Octave or MATLAB, the data must be stored in the .mat file as a cell
    array named ``data``.

    Each item of this cell array must be a cell array containing three cell
    arrays, the first containing the keys of **inputs**, the second the values
    of **inputs**, and the third the values of **outputs**.

    Here is an example to illustrate the expected use:

    In Octave or MATLAB, define the following variable::

        data = {{{'fun', 'dim', 'var'}, {'abs', 1, 1.3}, {[3., 2.2], 1.5}}, ...
                {{'fun', 'do_it'}, {'dgt', '_bool_True_'}, {[1.4, 1.2]}}};

    and save it using the save command::

        save('test_filename.mat', 'data', '-V6');

    In Python, using :func:`read_ref_mat` with::

        read_ref_mat('test_filename.mat')

    you get the following output::

        [({u'dim': 1, u'fun': u'abs', u'var': 1.3}, [array([3. ,  2.2]), 1.5]),
         ({u'do_it': True, u'fun': u'dgt'}, [array([1.4,  1.2])])]


    If ``adapt_types=True``, the following conversions are done for data that
    are an instance of numpy.ndarray:

    - Arrays with dtype numpy.unicode are converted to standard Python
      strings, and if their content is ``u'_bool_True_'`` or
      ``u'_bool_False_'`` they are converted to the corresponding boolean value
      (output of type bool)
    - If the array contains a single value, it is extracted from the array and
      converted to the appropriate Python object. If this value is a float
      having an integer value, it is converted to int.

    """
    data = scipy.io.loadmat(filename, chars_as_strings=True)["data"]
    res = list()
    for item in data[0, :]:
        if item[0, 0].size > 0:
            # the tolist in the following is used to convert the unicode array
            # to standard unicode string
            inputs = dict(
                (
                    (key[0].tolist(), val)
                    for key, val in zip(item[0, 0][0, :], item[0, 1][0, :])
                )
            )
        else:
            inputs = dict()

        outputs = item[0, 2][0, :].tolist()

        if squeeze_arrays:
            inputs = {key: _squeeze_array(val) for key, val in inputs.items()}
            outputs = [_squeeze_array(val) for val in outputs]
        if adapt_types:
            inputs = {key: _adapt_type(val) for key, val in inputs.items()}
            outputs = [_adapt_type(val) for val in outputs]
        res.append((inputs, outputs))
    return res



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of inverse FFT for real valued data computation

Ported from ltfat_2.1.0/fourier/ifftreal.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np


def ifftreal(c, N, dim=0):
    """Inverse FFT for real valued signals

    - Usage:

        | ``f = ifftreal(c, N)``
        | ``f = ifftreal(c, N, dim)``

    - Input parameters:

    :param numpy.ndarray c: Discrete Fourier coefficients of a real array for
        positive frequencies
    :param int N: IFFT length
    :param int dim: Axis over which to compute the IFFT. By default the first
        axis is used.

    - Output parameters:

    :returns: Reconstructed signal
    :rtype: numpy.ndarray

    ``ifftreal(c, N)`` computes an inverse FFT of the positive frequency
    Fourier coefficients **c**. The length **N** must always be specified,
    because the correct transform length cannot be determined from the
    size of **c**.

    ``ifftreal(c, N, dim)`` does the same along dimension **dim**.

    .. note::

        This Python port doesn't use the C core of LTFAT to implement the
        inverse FFT for real arrays as in the Octave version.

        Instead it direclty uses the implementation available in numpy.

        This can lead to sightly different results when comparing the results
        of the Octave and Python versions of ifftreal.

    .. seealso::  :func:`~ltfatpy.fourier.fftreal.fftreal`

    """

    return np.fft.irfft(c, N, dim)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of window calculation from descriptions

Ported from ltfat_2.1.0/comp/comp_window.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np
import copy

from ltfatpy.comp.arg_firwin import arg_firwin
from ltfatpy.comp.comp_pgauss import comp_pgauss
from ltfatpy.gabor.gabdual import gabdual
from ltfatpy.gabor.gabtight import gabtight
from ltfatpy.sigproc.firwin import firwin
from ltfatpy.fourier.psech import psech
from ltfatpy.sigproc.fir2long import fir2long
from ltfatpy.fourier.pgauss import pgauss


def comp_window(g, a, M, L=None):
    """Compute the window from numpy.ndarray, str or dictionary.

    - Usage:

        | ``(g, info) = comp_window(g, a, M, L)``

    - Input parameters:

    :param g: window parameters
    :type g: str or dict (with parameters of the window as keywords)
    :param int a: the length of time shift.
    :param int M: the number of channels.
    :param int L: the length of window.

    - Output parameters:

    :returns: ``(g, info)``
    :rtype: tuple
    :var numpy.ndarray g: the computed window
    :var dict info: the information dictionary

    It will compute the window from a text description or a dictionary
    containing additional parameters.

    .. note:: One field of the g dictionary must be the name of the window
        and should be define with the **name** key.

    .. note:: This function is the driving routine behind
        :func:`~ltfatpy.gabor.gabwin.gabwin`. Consider using this function
        instead.

    .. seealso:: :func:`~ltfatpy.gabor.gabwin.gabwin`, :func:`wilwin`

    .. note:: Basic discovery: some windows depend on *L*, and some windows
        help define *L*, so the calculation of *L* is window dependant.
    """
    # As this function can modify the content of g, we work on a copy to
    # avoid modifying the passed data
    g = copy.deepcopy(g)

    # Default values
    info = {}
    info['gauss'] = False
    info['wasrow'] = False
    info['isfir'] = False
    info['istight'] = False
    info['isdual'] = False
    errorMessage = "You must specify a length L if a window is represented"
    errorMessage += " as a text string or dictionary."

    # Manually get the list of FIR window names
    firwinnames = arg_firwin()

    # Create window if string was given as input.
    if isinstance(g, str):
        winname = g.lower()
        if winname in {'pgauss', 'gauss'}:
            if L is None:
                raise ValueError(errorMessage)
            g = comp_pgauss(L, a*M/L, 0, 0)
            info['gauss'] = True
            info['tfr'] = a*M/L
        elif winname in {'psech', 'sech'}:
            if L is None:
                raise ValueError(errorMessage)
            g = psech(L, a*M/L)[0]
            info['tfr'] = a*M/L
        elif winname in {'dualgauss', 'gaussdual'}:
            if L is None:
                raise ValueError(errorMessage)
            g = comp_pgauss(L, a*M/L, 0, 0)
            g = gabdual(g, a, M)
            info['isdual'] = True
            info['tfr'] = a*M/L
        elif winname in {"tight"}:
            if L is None:
                raise ValueError(errorMessage)
            g = gabtight(None, a, M, L)
            info['tfr'] = a*M/L
            info['istight'] = True
        elif winname in firwinnames:
            (g, firinfo) = firwin(winname, M, norm='2')
            info['isfir'] = True
            if firinfo['issqpu']:
                info['istight'] = True
        else:
            raise ValueError('Unknown window : ' + winname)
    # Create window if dictionary was given as input.
    elif isinstance(g, dict):
        if 'name' not in g:
            raise ValueError("the g dict must contain a key 'name'.")
#         if not isinstance(g['name'], str):
#             raise ValueError("the g['name'] value must be a string.")
        winname = g.pop('name')
        wintype = None
        if isinstance(winname, tuple):
            wintype = winname[0]
            if len(winname) > 1:
                winname = winname[1].lower()
            else:
                winname = 'gauss'
        elif isinstance(winname, str):
            winname = winname.lower()
        else:
            raise TypeError("the g['name'] value must be a string " +
                            "or a tuple.")
        if wintype is None:
            if winname in {'pgauss', 'gauss'}:
                if L is None:
                    raise ValueError(errorMessage)
                if 'tfr' not in g:
                    g, tfr = pgauss(L, a*M/L, **g)
                else:
                    g, tfr = pgauss(L, **g)
                info['tfr'] = tfr
                info['gauss'] = True
            elif winname in {'psech', 'sech'}:
                if L is None:
                    raise ValueError(errorMessage)
                if 'tfr' not in g:
                    g, tfr = psech(L,  a*M/L, **g)
                else:
                    g, tfr = psech(L, **g)
                info['tfr'] = tfr
            elif winname in firwinnames:
                if 'M' not in g:
                    raise ValueError("M has to be specified in the g " +
                                     "dictionary for fir windows.")
                else:
                    g, firinfo = firwin(winname, norm='energy', **g)
                info['isfir'] = True
                if firinfo['issqpu']:
                    info['istight'] = True
            else:
                raise ValueError('Unsupported window : ' + winname)
        elif wintype == 'dual':
            gorig = g.copy()
            gorig['name'] = winname
            if winname in firwinnames and 'M' not in gorig:
                gorig['M'] = M
            g, info['auxinfo'] = comp_window(gorig, a, M, L)
            g = gabdual(g, a, M, L)
            # gorig can be string or dict
            if info['auxinfo']['isfir'] and __test_isfir(gorig, M):
                info['isfir'] = True
            info['isdual'] = True
        elif wintype == 'tight':
            gorig = g.copy()
            gorig['name'] = winname
            if winname in firwinnames and 'M' not in gorig:
                gorig['M'] = M
            g, info['auxinfo'] = comp_window(gorig, a, M, L)
            g = gabtight(g, a, M, L)
            # The same as in dual?
            if info['auxinfo']['isfir'] and __test_isfir(gorig, M):
                info['isfir'] = True
            info['istight'] = True

    if isinstance(g, np.ndarray):
        if g.ndim > 1 and g.shape[1] > 1:
            if g.shape[0] == 1:
                # g was a row vector.
                g = g.squeeze()
                info['wasrow'] = True

        if g.shape[0] % M > 0:
            # Zero-extend the window to a multiple of M
            g = fir2long(g, int(np.ceil(g.shape[0] / M) * M))

        # Information to be determined post creation.
        info['wasreal'] = np.issubdtype(g.dtype, np.floating)
        info['gl'] = g.shape[0]
        if L is not None and info['gl'] < L:
            info['isfir'] = True
    return (g, info)


def __test_isfir(gorig, M):
    """ Internal function that tests if original window is FIR.

    Dual window is FIR if length of the original window is <= M. This is true
    if the length was not explicitly defined and gorig is a string.

    - Input parameters:

    :param gorig: window parameters
    :type gorig: dict (with parameters of the window as keywords)
    :param int M: the number of channels.

    - Output parameters:

    :rtype: boolean
    """
    if (isinstance(gorig, dict) and gorig):
        if ('M' in gorig) and (gorig['M'] <= M):
            return True
    if isinstance(gorig, str):
        return True
    return False



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the comp_sigreshape_pre and comp_sigreshape_post functions

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import unittest
import random
import numpy as np
from ltfatpy.comp.comp_sigreshape_pre import comp_sigreshape_pre as cpre
from ltfatpy.comp.comp_sigreshape_post import comp_sigreshape_post as cpost
import functools


class TestCompSigReshape(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print("Start TestCompSigReshape")

    # Called after the tests.
    def tearDown(self):
        print("Test done")

    def test_default(self):
        self.assertRaises(ValueError, cpre, "toto", 10)
        self.assertRaises(ValueError, cpre, np.empty((0,)), 10)
        self.assertRaises(ValueError, cpre, np.ones((10, 10, 2)), 1)

    def test_onedim(self):
        """One channel"""
        L = random.randint(5, 200)
        f = np.arange(0, L, dtype=np.float64)
        (fin, fl, W, wasrow, remembershape) = cpre(f, 0)
        fres = cpost(fin, fl, wasrow, remembershape)
        mess = "\nfl = {:d}, W = {:d}, wasrow = {:d},remembershape.shape = "
        mess += str(remembershape) + ", f.shape = " + str(f.shape)
        mess += ", fres.shape = " + str(fres.shape)
        mess = mess.format(fl, W, wasrow)
        np.testing.assert_array_equal(f, fres, mess)
        L = random.randint(5, 200)
        f = np.arange(0, L, dtype=np.float64)
        f.resize((1, L))
        (fin, fl, W, wasrow, remembershape) = cpre(f, 0)
        fres = cpost(fin, fl, wasrow, remembershape)
        mess = "\nfl = {:d}, W = {:d}, wasrow = {:d},remembershape.shape = "
        mess += str(remembershape) + ", f.shape = " + str(f.shape)
        mess += ", fres.shape = " + str(fres.shape)
        mess = mess.format(fl, W, wasrow)
        np.testing.assert_array_equal(f, fres, mess)

    def test_twodim(self):
        """Many channels signals"""
        shapef = tuple([random.randint(5, 20) for _ in range(2)])
        L = functools.reduce(lambda x, y: x * y, shapef)
        f = np.arange(0, L, dtype=np.complex128)
        f = f.reshape(shapef)
        (fin, fl, W, wasrow, remembershape) = cpre(f, 1)
        fres = cpost(fin, fl, wasrow, remembershape)
        mess = "\nfl = {:d}, W = {:d}, wasrow = {:d},remembershape.shape = "
        mess += str(remembershape) + ", f.shape = " + str(f.shape)
        mess += ", fres.shape = " + str(fres.shape)
        mess = mess.format(fl, W, wasrow)
        np.testing.assert_array_equal(f, fres, mess)

    def test_multidim(self):
        """Multidimensionnal signals"""
        shapef = tuple([random.randint(5, 20) for _ in range(4)])
        L = np.prod(shapef)
        f = np.arange(0, L, dtype=np.complex128)
        f = f.reshape(shapef)
        (fin, fl, W, wasrow, remembershape) = cpre(f, 5)
        fres = cpost(fin, fl, wasrow, remembershape)
        mess = "\nfl = {:d}, W = {:.2f}, wasrow = {:d},remembershape.shape = "
        mess += str(remembershape) + ", f.shape = " + str(f.shape)
        mess += ", fres.shape = " + str(fres.shape)
        mess = mess.format(fl, W, wasrow)
        np.testing.assert_array_equal(f, fres, mess)


if __name__ == "__main__":
    suite = unittest.TestLoader().loadTestsFromTestCase(TestCompSigReshape)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module to find Gabor parameters to generate image

Ported from ltfat_2.1.0/gabor/gabimagepars.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.tools.lcm import lcm


def gabimagepars(Ls, x, y):
    """Find Gabor parameters to generate image

    - Usage:

        | ``(a, M, L, N, Ngood) = gabimagepars(Ls, x, y)``

    - Input parameters:

    :param int Ls: Signal length
    :param int x: Approximate number of pixels in the time direction
    :param int y: Number of pixels in the frequency direction

    - Output parameters:

    :returns: ``(a, M, L, N, Ngood)``
    :rtype: tuple

    :var int a: Length of time shift
    :var int M: Number of frequency channels
    :var int L: Length of transform to do
    :var int N: Total number of time steps
    :var int Ngood: Number of time steps (columns in the coefficients
        matrix) that contain relevant information. The columns from
        ``Ngood-1`` until ``N-1`` only contain information from a
        zero-extension of the signal.

    ``(a, M, L, N, Ngood) = gabimagepars(Ls, x, y)`` will compute a reasonable
    set of parameters **a**, **M** and **L** to produce a nice Gabor 'image' of
    a signal of length **Ls**.

    If you use this function to calculate a grid size for analysis of a
    real-valued signal (using :func:`~ltfatpy.gabor.dgtreal.dgtreal`), please
    input twice of the desired size **y**. This is because
    :func:`~ltfatpy.gabor.dgtreal.dgtreal` only returns half as many
    coefficients in the frequency direction as :func:`~ltfatpy.gabor.dgt.dgt`.

    For this function to work properly, the specified numbers for **x** and
    **y** must not be large prime numbers.

    - Example:

        We wish to compute a Gabor image of a real valued signal ``f``
        of length 7500. The image should have an approximate resolution of
        600 x 800 pixels:

        >>> from matplotlib.pyplot import show
        >>> from ltfatpy import linus, gabimagepars, dgtreal, plotdgtreal
        >>> f, fs = linus()
        >>> f = f[4000:4000+7500]
        >>> a, M, L, N, Ngood = gabimagepars(7500, 800, 2*600)
        >>> c = dgtreal(f, 'gauss', a, M)[0]
        >>> _ = plotdgtreal(c, a, M, fs=fs, dynrange=90)
        >>> show()

        The size of ``c`` is ``(M/2)+1 x N`` equal to 601 x 700 pixels.

    .. image:: images/gabimagepars.png
       :width: 600px
       :alt: Gabor image of f
       :align: center

    .. seealso:: :func:`~ltfatpy.gabor.dgt.dgt`,
                 :func:`~ltfatpy.gabor.dgtreal.dgtreal`,
                 :func:`~ltfatpy.gabor.sgram.sgram`
    """

    # Note: There is an inaccuracy in the help of the function gabimagepars in
    # ltfat 2.1.0 for Octave.
    # This inaccuracy concerns the size of the resulting coefficients in the
    # example. See the description and confirmation here:
    # http://sourceforge.net/p/ltfat/bugs/120/
    # This inaccuracy is corrected in the docstring of this Python port.

    if min(x, y) > Ls:
        # Small values case, just do an STFT
        M = Ls
        N = Ls
        a = 1
        Ngood = N
        L = Ls
    else:
        # Set M and N to be what the user specified
        M = y
        N = x

        # Determine the minimum transform size.
        K = lcm(M, N)

        # This L is good, but is it not the same as DGT will choose.
        Llong = np.ceil(Ls/K)*K

        # Fix a from the long L
        a = int(Llong/N)

        # Now we have fixed a and M, so we can use the standard method of
        # choosing L
        Lsmallest = lcm(a, M)
        L = int(np.ceil(Ls/Lsmallest)*Lsmallest)

        # We did not get N as desired.
        N = int(L/a)

        # Number of columns to display
        Ngood = int(np.ceil(Ls/a))

        if M <= a:
            raise ValueError(
                'Cannot generate a frame, the signal is too long '
                'as compared to the size of the image. Increase x and y.')

    return (a, M, L, N, Ngood)


if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########

"""The **ltfatpy** package is a partial Python port of the
`Large Time/Frequency Analysis Toolbox (LTFAT)
<http://ltfat.sourceforge.net/>`_, a MATLAB®/Octave toolbox for working with
time-frequency analysis and synthesis.

It is intended both as an educational and a computational tool.

The package provides a large number of linear transforms including Gabor
transforms along with routines for constructing windows (filter prototypes)
and routines for manipulating coefficients.

The original LTFAT Toolbox for MATLAB®/Octave is developed at
`CAHR <http://www.dtu.dk/centre/cahr/English.aspx>`_, Technical
University of Denmark, `ARI <http://www.kfs.oeaw.ac.at>`_, Austrian Academy
of Sciences and `I2M <http://www.i2m.univ-amu.fr>`_, Aix-Marseille Université.

The Python port is developed at
`LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_, as a
`LIF <http://www.lif.univ-mrs.fr/>`_ (now `LIS <http://www.lis-lab.fr/>`_)
and `I2M <https://www.i2m.univ-amu.fr/>`__ project, Aix-Marseille Université.

This package, as well as the original LTFAT toolbox, is Free software, released
under the GNU General Public License (GPLv3).
"""

from ltfatpy.comp.arg_firwin import arg_firwin
from ltfatpy.comp.assert_groworder import assert_groworder
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post
from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.comp_dct import comp_dct
from ltfatpy.comp.comp_dst import comp_dst
from ltfatpy.comp.comp_gabdual_long import comp_gabdual_long
from ltfatpy.comp.comp_gabtight_long import comp_gabtight_long
from ltfatpy.comp.comp_hermite import comp_hermite
from ltfatpy.comp.comp_hermite_all import comp_hermite_all
from ltfatpy.comp.comp_isepdgt import comp_isepdgt
from ltfatpy.comp.comp_isepdgtreal import comp_isepdgtreal
from ltfatpy.comp.comp_pgauss import comp_pgauss
from ltfatpy.comp.comp_sepdgt import comp_sepdgt
from ltfatpy.comp.comp_sepdgtreal import comp_sepdgtreal
from ltfatpy.comp.comp_sigreshape_post import comp_sigreshape_post
from ltfatpy.comp.comp_sigreshape_pre import comp_sigreshape_pre
from ltfatpy.comp.comp_window import comp_window
from ltfatpy.comp.gabpars_from_windowsignal import gabpars_from_windowsignal
from ltfatpy.fourier.dcti import dcti
from ltfatpy.fourier.dctii import dctii
from ltfatpy.fourier.dctiii import dctiii
from ltfatpy.fourier.dctiv import dctiv
from ltfatpy.fourier.dft import dft
from ltfatpy.fourier.dsti import dsti
from ltfatpy.fourier.dstii import dstii
from ltfatpy.fourier.dstiii import dstiii
from ltfatpy.fourier.dstiv import dstiv
from ltfatpy.fourier.fftindex import fftindex
from ltfatpy.fourier.fftreal import fftreal
from ltfatpy.fourier.fftresample import fftresample
from ltfatpy.fourier.idft import idft
from ltfatpy.fourier.ifftreal import ifftreal
from ltfatpy.fourier.isevenfunction import isevenfunction
from ltfatpy.fourier.middlepad import middlepad
from ltfatpy.fourier.pderiv import pderiv
from ltfatpy.fourier.pgauss import pgauss
from ltfatpy.fourier.pherm import pherm
from ltfatpy.fourier.psech import psech
from ltfatpy.gabor.dgt import dgt
from ltfatpy.gabor.dgtlength import dgtlength
from ltfatpy.gabor.dgtreal import dgtreal
from ltfatpy.gabor.gabdual import gabdual
from ltfatpy.gabor.gabframediag import gabframediag
from ltfatpy.gabor.gabimagepars import gabimagepars
from ltfatpy.gabor.gabphasegrad import gabphasegrad
from ltfatpy.gabor.gabtight import gabtight
from ltfatpy.gabor.gabwin import gabwin
from ltfatpy.gabor.idgt import idgt
from ltfatpy.gabor.idgtreal import idgtreal
from ltfatpy.gabor.instfreqplot import instfreqplot
from ltfatpy.gabor.phaselock import phaselock
from ltfatpy.gabor.phaseplot import phaseplot
from ltfatpy.gabor.phaseunlock import phaseunlock
from ltfatpy.gabor.plotdgt import plotdgt
from ltfatpy.gabor.plotdgtreal import plotdgtreal
from ltfatpy.gabor.s0norm import s0norm
from ltfatpy.gabor.sgram import sgram
from ltfatpy.gabor.tfplot import tfplot
from ltfatpy.signals.greasy import greasy
from ltfatpy.signals.gspi import gspi
from ltfatpy.signals.linus import linus
from ltfatpy.sigproc.fir2long import fir2long
from ltfatpy.sigproc.firkaiser import firkaiser
from ltfatpy.sigproc.firwin import firwin
from ltfatpy.sigproc.groupthresh import groupthresh
from ltfatpy.sigproc.largestn import largestn
from ltfatpy.sigproc.largestr import largestr
from ltfatpy.sigproc.long2fir import long2fir
from ltfatpy.sigproc.normalize import normalize
from ltfatpy.sigproc.rms import rms
from ltfatpy.sigproc.thresh import thresh
from ltfatpy.tools.lcm import lcm
from ltfatpy.tools.postpad import postpad

__all__ = ["arg_firwin", "assert_groworder", "assert_sigreshape_post",
           "assert_sigreshape_pre", "comp_dct", "comp_dst",
           "comp_gabdual_long", "comp_gabtight_long", "comp_hermite",
           "comp_hermite_all", "comp_isepdgt", "comp_isepdgtreal",
           "comp_pgauss", "comp_sepdgt", "comp_sepdgtreal",
           "comp_sigreshape_post", "comp_sigreshape_pre", "comp_window",
           "gabpars_from_windowsignal", "dcti", "dctii", "dctiii", "dctiv",
           "dft", "dsti", "dstii", "dstiii", "dstiv", "fftindex", "fftreal",
           "fftresample", "idft", "ifftreal", "isevenfunction", "middlepad",
           "pderiv", "pgauss", "pherm", "psech", "dgt", "dgtlength", "dgtreal",
           "gabdual", "gabframediag", "gabimagepars", "gabphasegrad",
           "gabtight", "gabwin", "idgt", "idgtreal", "instfreqplot",
           "phaselock", "phaseplot", "phaseunlock", "plotdgt", "plotdgtreal",
           "s0norm", "sgram", "tfplot", "greasy", "gspi", "linus", "fir2long",
           "firkaiser", "firwin", "groupthresh", "largestn", "largestr",
           "long2fir", "normalize", "rms", "thresh", "lcm", "postpad"]


__version__ = "1.1.2"



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


cdef enum dgt_phasetype:
    FREQINV = 0
    TIMEINV = 1

ctypedef size_t ltfatInt

# Values extracted from fftw3.h
cdef enum dct_kind:
    DCTI = 3
    DCTIII = 4
    DCTII = 5
    DCTIV = 6

# Values extracted from fftw3.h
cdef enum dst_kind:
    DSTI = 7
    DSTIII = 8
    DSTII = 9
    DSTIV = 10


# pgauss functions declarations
cdef extern void pgauss_d(const ltfatInt L, const double w, const double c_t,
                          double * g)

cdef extern void pgauss_cmplx_d(const ltfatInt L, const double w,
                                const double c_t, const double c_f,
                                const double complex * g)

# DGT functions declarations
cdef extern void dgt_long_cd(const double complex * f,
                             const double complex * g,
                             const ltfatInt L, const ltfatInt W,
                             const ltfatInt a, const ltfatInt M,
                             const dgt_phasetype ptype, double complex * cout)

cdef extern void dgt_long_d(const double * f, const double * g,
                            const ltfatInt L, const ltfatInt W,
                            const ltfatInt a, const ltfatInt M,
                            const dgt_phasetype ptype, double complex * cout)

cdef extern void dgtreal_long_d(const double * f, const double * g,
                                const ltfatInt L, const ltfatInt W,
                                const ltfatInt a, const ltfatInt M,
                                const dgt_phasetype ptype,
                                double complex * cout)

cdef extern void dgt_fb_cd(const double complex * f, const double complex * g,
                           const ltfatInt L, const ltfatInt gl,
                           const ltfatInt W, const ltfatInt a,
                           const ltfatInt M, const dgt_phasetype ptype,
                           double complex * cout)

cdef extern void dgt_fb_d(const double * f, const double * g,
                          const ltfatInt L, const ltfatInt gl,
                          const ltfatInt W, const ltfatInt a,
                          const ltfatInt M, const dgt_phasetype ptype,
                          double complex * cout)

cdef extern void dgtreal_fb_d(const double * f, const double * g,
                              const ltfatInt L, const ltfatInt gl,
                              const ltfatInt W, const ltfatInt a,
                              const ltfatInt M, const dgt_phasetype ptype,
                              double complex * cout)

cdef extern void idgt_fb_d(const double complex * F, const double complex * g,
                           const ltfatInt L, const ltfatInt gl,
                           const ltfatInt W, const ltfatInt a,
                           const ltfatInt M, const dgt_phasetype ptype,
                           double complex * f)

cdef extern void idgtreal_fb_d(const double complex * F, const double * g,
                               const ltfatInt L, const ltfatInt gl,
                               const ltfatInt W,
                               const ltfatInt a, const ltfatInt M,
                               const dgt_phasetype ptype, double * f)

cdef extern void idgt_long_d(const double complex * F,
                             const double complex * g,
                             const ltfatInt L, const ltfatInt W,
                             const ltfatInt a, const ltfatInt M,
                             const dgt_phasetype ptype,
                             double complex * f)

cdef extern void idgtreal_long_d(const double complex * F, const double * g,
                                 const ltfatInt L, const ltfatInt W,
                                 const ltfatInt a, const ltfatInt M,
                                 const dgt_phasetype ptype, double * f)

cdef extern void gabdual_long_d(const double * g,
                                const ltfatInt L, const ltfatInt R,
                                const ltfatInt a,
                                const ltfatInt M, double * gd)

cdef extern void gabdual_long_cd(const double complex * g,
                                 const ltfatInt L, const ltfatInt R,
                                 const ltfatInt a,
                                 const ltfatInt M, double complex * gd)

cdef extern void gabtight_long_d(const double * g,
                                 const ltfatInt L, const ltfatInt R,
                                 const ltfatInt a, const ltfatInt M,
                                 double * gd)

cdef extern void gabtight_long_cd(const double complex * g,
                                  const ltfatInt L, const ltfatInt R,
                                  const ltfatInt a, const ltfatInt M,
                                  double complex * gd)

cdef extern void dct_d(const double * f, const ltfatInt L, const ltfatInt W,
                       double * cout, const dct_kind kind)

cdef extern void dct_cd(const double complex * f, const ltfatInt L,
                        const ltfatInt W, double complex * cout,
                        const dct_kind kind)

cdef extern void dst_d(const double * f, const ltfatInt L, const ltfatInt W,
                       double * cout, const dst_kind kind)

cdef extern void dst_cd(const double complex * f, const ltfatInt L,
                        const ltfatInt W, double complex * cout,
                        const dst_kind kind)




# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of coefficient thresholding

Ported from ltfat_2.1.0/sigproc/thresh.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np


def thresh(xi, lamb, thresh_type='hard'):
    """Coefficient thresholding

    - Usage:

        | ``(xo, N) = thresh(xi, lamb)``
        | ``(xo, N) = thresh(xi, lamb, thresh_type)``

    - Input parameters:

    :param numpy.ndarray xi: Input array
    :param lamb: Threshold
    :type lamb: float or numpy.ndarray
    :param str thresh_type: Optional flag specifying the type of thresholding
        (see possible values below)

    - Output parameters:

    :returns: ``(xo, N)``
    :rtype: tuple

    :var numpy.ndarray xo: Array of the same shape as **xi**
        containing data from **xi** after thresholding
    :var int N: Number of coefficients kept

    ``thresh(xi, lamb)`` will perform hard thresholding on **xi**, i.e. all
    elements with absolute value less than scalar **lamb** will be set to zero.

    ``thresh(xi, lamb, 'soft')`` will perform soft thresholding on **xi**, i.e.
    **lamb** will be substracted from the absolute value of every element of
    **xi**.

    The lamb parameter can also be a vector with number of elements
    equal to ``xi.size`` or it can be a numpy array of the same shape
    as **xi**. **lamb** is then applied element-wise and in a column major
    order if **lamb** is a vector.

    The parameter **thresh_type** can take the following values:
        ============ ======================================================
        ``'hard'``   Perform hard thresholding. This is the default.

        ``'wiener'`` Perform empirical Wiener shrinkage. This is in between
                     soft and hard thresholding.

        ``'soft'``   Perform soft thresholding.
        ============ ======================================================

    The function ``wthresh`` in the Matlab Wavelet toolbox implements some of
    the same functionality.

    - Example:

        The following code produces a plot to demonstrate the difference
        between hard and soft thresholding for a simple linear input:

        >>> import numpy as np
        >>> import matplotlib.pyplot as plt
        >>> from ltfatpy.sigproc.thresh import thresh
        >>> t = np.linspace(-4, 4, 100)
        >>> _ = plt.plot(t, thresh(t, 1., 'soft')[0], 'r',
        ... t, thresh(t, 1., 'hard')[0], '.b',
        ... t, thresh(t, 1., 'wiener')[0], '--g')
        >>> _ = plt.legend(('Soft thresh.', 'Hard thresh.',  'Wiener thresh.'),
        ... loc='upper left')
        >>> plt.show()

    .. image:: images/thresh.png
       :width: 700px
       :alt: thresh image
       :align: center

    .. seealso::
        :func:`~ltfatpy.sigproc.largestr.largestr`,
        :func:`~ltfatpy.sigproc.largestn.largestn`

    - References:
        :cite:`lim1979enhancement,ghael1997improved`
    """

    # Note: This function doesn't support the handling of sparse matrices
    # available in the Octave version. Only full numpy arrays are supported in
    # input and output.

    error_msg = ('lamb must be a float or a numpy vector with '
                 'lamb.size == xi.size or whatever shape xi has such that '
                 'lamb.shape == xi.shape')

    if not (isinstance(lamb, float) or isinstance(lamb, np.ndarray)):
        raise TypeError(error_msg)

    if isinstance(lamb, np.ndarray):  # lamb is not scalar
        if lamb.size != xi.size:
            # lamb does not have the same number of elements
            raise ValueError(error_msg)

        # Reshape lamb if it is a vector
        if lamb.shape != xi.shape:
            lamb = lamb.reshape(xi.shape, order='F')

    # Dense case (this Python port doesn't handle the sparse matrix case)
    xo = np.zeros(xi.shape, dtype=xi.dtype)

    # Create a mask with a value of 1 for non-zero elements. For full
    # matrices, this is faster than the significance map.

    if thresh_type == 'hard':
        mask = abs(xi) >= lamb
        N = np.count_nonzero(mask)
        xo = xi * mask

    elif thresh_type == 'soft':
        # In the following lines, the +0 is significant: It turns
        # -0 into +0, oh! the joy of numerics.
        # Note: It is not sure that the "+0." needed in Octave is also needed
        # in Python, but it is kept here for safety.
        xa = abs(xi)-lamb
        mask = xa >= 0.
        xo = (mask*xa + 0.) * np.sign(xi)
        N = np.count_nonzero(mask) - np.count_nonzero(xa == 0.)

    elif thresh_type == 'wiener':
        with np.errstate(divide='ignore'):
            # NOTE: divide by 0 warnings are ignored because they are handled
            # below
            xa = lamb / abs(xi)
        xa[np.isinf(xa)] = 0
        xa = 1. - xa**2
        mask = xa > 0
        xo = xi * xa * mask
        N = np.count_nonzero(mask)

    return (xo, N)

if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of N largest coefficients extraction

Ported from ltfat_2.1.0/sigproc/largestn.m

.. moduleauthor:: Florent Jaillet
"""

import numpy as np

from ltfatpy.sigproc.thresh import thresh


def largestn(xi, N, thresh_type='hard'):
    """Keep N largest coefficients

    - Usage:

        | ``(xo, Nout) = largestn(xi, N)``
        | ``(xo, Nout) = largestn(xi, N, thresh_type)``

    - Input parameters:

    :param numpy.ndarray xi: Input array
    :param int N: Number of kept coefficients
    :param str thresh_type: Optional flag specifying the type of thresholding
        (see possible values below)

    - Output parameters:

    :returns: ``(xo, Nout)``
    :rtype: tuple

    :var numpy.ndarray xo: Array of the same shape as **xi** keeping
        the **N** largest coefficients
    :var int Nout: Number of coefficients kept

    The parameter **thresh_type** can take the following values:
        ============ ======================================================
        ``'hard'``   Perform hard thresholding. This is the default.

        ``'wiener'`` Perform empirical Wiener shrinkage. This is in between
                     soft and hard thresholding.

        ``'soft'``   Perform soft thresholding.
        ============ ======================================================

    If the coefficients represents a signal expanded in an orthonormal
    basis then this will be the best N-term approximation.

    .. note::
        If soft- or Wiener thresholding is selected, only ``N-1``
        coefficients will actually be returned. This is caused by the Nth
        coefficient being set to zero.

    .. seealso::
        :func:`~ltfatpy.sigproc.largestr.largestr`

    - References:
        :cite:`ma98`
    """

    if not isinstance(N, int):
        raise TypeError('N must be an int.')

    # Sort the absolute values of the coefficients.
    sxi = np.sort(abs(xi.flatten()))

    # Find the coefficient sitting at position N through the array,
    # and use this as a threshing value.
    if N <= 0:
        # Choose a thresh value higher than max
        lamb = sxi[-1] + 1.
    else:
        lamb = sxi[-N]

    xo, Nout = thresh(xi, lamb, thresh_type)

    return (xo, Nout)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""This module contains interface functions for the LTFAT computed
versions of periodized Gaussian calculation.

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import cython
from cython.view cimport array as cvarray
from libc.stdlib cimport malloc, free
import numpy as np

from ltfat cimport ltfatInt, pgauss_d, pgauss_cmplx_d

# don’t check for out-of-bounds indexing.
@cython.boundscheck(False)
# assume no negative indexing.
@cython.wraparound(False)
cdef comp_pgauss_real(ltfatInt L, double w, double c_t,
                      double[:] out):
    """ Internal function, do not use it """
    pgauss_d(L, w, c_t, & out[0])

# don’t check for out-of-bounds indexing.
@cython.boundscheck(False)
# assume no negative indexing.
@cython.wraparound(False)
cdef comp_pgauss_cplx(ltfatInt L, double w, double c_t,
                      double c_f, double complex[:] out):
    """ Internal function, do not use it """
    pgauss_cmplx_d(L, w, c_t, c_f, & out[0])

cpdef comp_pgauss(L, w=1.0, c_t=0, c_f=0):
    """Return the computed sampled, periodized Gaussian.

    This is a computational subroutine, do not call it directly, use
    :func:`~ltfatpy.fourier.pgauss.pgauss` instead.
    """
    if c_f == 0:
        out = np.ndarray((L), dtype=np.float64)
        comp_pgauss_real(L, w, c_t, out)
        return out
    out = np.ndarray((L), dtype=np.complex128)
    comp_pgauss_cplx(L, w, c_t, c_f, out)
    return out



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of sampling of all Hermite function computation

Ported from ltfat_2.1.0/comp/comp_hermite_all.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np


def comp_hermite_all(n, x):
    """ Compute all Hermite functions up to an order

    - Usage:

        | ``y = comp_hermite_all(n, x)``

    This function evaluates the Hermite functions
    of degree 0 through n-1 at the vector x.
    The functions are normalized to have the :math:`L^2` norm
    on :math:`]-\\infty,\\infty[` equal to one. No effort is made to
    avoid unerflow during recursion.

    - Input parameters:

    :param int n: the number of Hermite functions
    :param numpy.ndarray x: the vector of arguments

    - Output parameters:

    :return numpy.ndarray y: the values of the first n Hermite functions at
                             the nodes x
    """
    if not isinstance(x, np.ndarray) or len(x.shape) > 1:
        raise TypeError("x should be a numpy array of dimension 1")
    rt = 1 / np.sqrt(np.sqrt(np.pi))

    # conducting the recursion.
    y = np.zeros((len(x), n))

    if n == 0:
        y = rt * np.exp(-0.5 * x**2)
    else:
        y[:, 0] = rt * np.exp(-0.5 * x**2)
        if n > 1:
            y[:, 1] = rt * np.sqrt(2) * x * np.exp(-0.5 * x**2)
            for k in range(2, n):
                y[:, k] = np.sqrt(2)*x*y[:, k-1] - np.sqrt(k-1)*y[:, k-2]
                y[:, k] = y[:, k] / np.sqrt(k)
    return y



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of phase gradient computation

Ported from ltfat_2.1.0/gabor/gabphasegrad.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.comp.comp_sigreshape_pre import comp_sigreshape_pre
from ltfatpy.gabor.dgtlength import dgtlength
from ltfatpy.gabor.gabwin import gabwin
from ltfatpy.tools.postpad import postpad
from ltfatpy.fourier.fftindex import fftindex
from ltfatpy.comp.comp_sepdgt import comp_sepdgt
from ltfatpy.fourier.pderiv import pderiv


def gabphasegrad(method, *args, **kwargs):
    """Phase gradient of the discrete Gabor transform

    - Usage:

        | ``(tgrad, fgrad, c) = gabphasegrad('dgt', f, g, a, M, L=None)``
        | ``(tgrad, fgrad) = gabphasegrad('phase', cphase, a)``
        | ``(tgrad, fgrad) = gabphasegrad('abs', s, g, a, difforder=4)``

    - Input parameters:

    :param str method: Method used to compute the phase gradient, see the
        possible values below
    :param numpy.ndarray f: (defined if ``method='dgt'``) Input signal
    :param numpy.ndarray cphase: (defined if ``method='phase'``) Phase of a
        :func:`~ltfatpy.gabor.dgt.dgt` of the signal
    :param numpy.ndarray s: (defined if ``method='abs'``) Spectrogram of the
        signal
    :param numpy.ndarray g: (defined if ``method='dgt'`` or ``method='phase'``)
        Window function
    :param int a: (defined if ``method='dgt'`` or ``method='phase'`` or
        ``method='abs'``) Length of time shift
    :param int M: (defined if ``method='dgt'``) Number of channels
    :param int L: (defined if ``method='dgt'``, optional) Length of transform
        to do
    :param int difforder: (defined if ``method='abs'``, optional) Order of the
        centered finite difference scheme used to perform the needed numerical
        differentiation

    - Output parameters:

    :returns: ``(tgrad, fgrad, c)`` if ``method='dgt'``, or ``(tgrad, fgrad)``
        if ``method='phase'`` or ``method='abs'``
    :rtype: tuple

    :var numpy.ndarray tgrad: Instantaneous frequency
    :var numpy.ndarray fgrad: Local group delay
    :var numpy.ndarray c: Gabor coefficients

    ``gabphasegrad`` computes the time-frequency gradient of the phase of the
    :func:`~ltfatpy.gabor.dgt.dgt` of a signal. The derivative in time
    **tgrad** is the instantaneous frequency while the frequency derivative
    **fgrad** is the local group delay.

    **tgrad** and **fgrad** measure the deviation from the current time and
    frequency, so a value of zero means that the instantaneous frequency is
    equal to the center frequency of the considered channel.

    **tgrad** is scaled such that distances are measured in samples. Similarly,
    **fgrad** is scaled such that the Nyquist frequency (the highest possible
    frequency) corresponds to a value of ``L/2``.

    The computation of **tgrad** and **fgrad** is inaccurate when the absolute
    value of the Gabor coefficients is low. This is due to the fact the the
    phase of complex numbers close to the machine precision is almost
    random. Therefore, **tgrad** and **fgrad** may attain very large random
    values when ``abs(c)`` is close to zero.

    The computation can be done using three different methods:

        =========== ===========================================================
        ``'dgt'``   Directly from the signal.

        ``'phase'`` From the phase of a :func:`~ltfatpy.gabor.dgt.dgt` of the
                    signal. This is the classic method used in the phase
                    vocoder.

        ``'abs'``   From the absolute value of the
                    :func:`~ltfatpy.gabor.dgt.dgt`. Currently this method works
                    only for Gaussian windows.
        =========== ===========================================================

    ``(tgrad, fgrad, c) = gabphasegrad('dgt', f, g, a, M)`` computes the
    time-frequency gradient using a :func:`~ltfatpy.gabor.dgt.dgt` of the
    signal **f**. The :func:`~ltfatpy.gabor.dgt.dgt` is computed using the
    window **g** on the lattice specified by the time shift **a** and the
    number of channels **M**. The algorithm used to perform this calculation
    computes several DGTs, and therefore this routine takes the exact same
    input parameters as :func:`~ltfatpy.gabor.dgt.dgt`.

    The window **g** may be specified as in :func:`~ltfatpy.gabor.dgt.dgt`. If
    the window used is ``'gauss'``, the computation will be done by a faster
    algorithm.

    ``(tgrad, fgrad, c) = gabphasegrad('dgt', f, g, a, M)`` additionally
    returns the Gabor coefficients ``c``, as they are always computed as a
    byproduct of the algorithm.

    ``(tgrad, fgrad) = gabphasegrad('phase', cphase, a)`` computes the phase
    gradient from the phase **cphase** of a :func:`~ltfatpy.gabor.dgt.dgt` of
    the signal. The original :func:`~ltfatpy.gabor.dgt.dgt` from which the
    phase is obtained must have been computed using a time-shift of **a**.

    ``(tgrad, fgrad) = gabphasegrad('abs', s, g, a)`` computes the phase
    gradient from the spectrogram **s**. The spectrogram must have been
    computed using the window **g** and time-shift **a**.

    ``(tgrad, fgrad) = gabphasegrad('abs', s, g, a, difforder=ord)`` uses a
    centered finite difference scheme of order ``ord`` to perform the needed
    numerical differentiation. Default is to use a 4th order scheme.

    Currently the 'abs' method only works if the window **g** is a Gaussian
    window specified as a string or cell array.

    .. seealso:: :func:`resgram`, :func:`gabreassign`,
                 :func:`~ltfatpy.gabor.dgt.dgt`

    - References:
        :cite:`aufl95,cmdaaufl97,fl65`
    """

    # NOTE: This function doesn't support the parameter lt (lattice type)
    # supported by the corresponding octave function and the lattice used is
    # seperable (square lattice lt = (0, 1)).

    # NOTE: As in the octave version of this function, if needed, the
    # undocumented optional keyword minlvl is available when using method=dgt.
    # So it can be passed using a call of the following form:
    # (tgrad, fgrad, c) = gabphasegrad('dgt', f, g, a, M, minlvl=val)

    if not isinstance(method, str):
        raise TypeError('First argument must be a str containing the method '
                        'name, "dgt", "phase" or "abs".')

    method = method.lower()

    if method == 'dgt':
        # ---------------------------  DGT method ------------------------

        (f, g, a, M) = args

        if 'L' in kwargs:
            L = kwargs['L']
        else:
            L = None

        if 'minlvl' in kwargs:
            minlvl = kwargs['minlvl']
        else:
            minlvl = np.finfo(np.float64).tiny

        # # ----- step 1 : Verify f and determine its length -------
        # Change f to correct shape.

        f, Ls, W, wasrow, remembershape = comp_sigreshape_pre(f, 0)

        # # ------ step 2: Verify a, M and L
        if not L:
            # ----- step 2b : Verify a, M and get L from the signal length f---
            L = dgtlength(Ls, a, M)
        else:
            # ----- step 2a : Verify a, M and get L
            Luser = dgtlength(L, a, M)
            if Luser != L:
                raise ValueError('Incorrect transform length L = {0:d} '
                                 'specified. Next valid length is L = {1:d}. '
                                 'See the help of dgtlength for the '
                                 'requirements.'.format(L, Luser))

        # # ----- step 3 : Determine the window
        g, info = gabwin(g, a, M, L)

        if L < info['gl']:
            raise ValueError('Window is too long.')

        # # ----- step 4: final cleanup ---------------

        f = postpad(f, L)

        # # ------ algorithm starts --------------------

        # Compute the time weighted version of the window.
        hg = fftindex(L) * g

        # The computation done this way is insensitive to whether the dgt is
        # phaselocked or not.
        c = comp_sepdgt(f, g, a, M, 0)

        c_h = comp_sepdgt(f, hg, a, M, 0)

        c_s = np.abs(c)**2

        # Remove small values because we need to divide by c_s
        c_s = np.maximum(c_s, minlvl*np.max(c_s))

        # Compute the group delay
        fgrad = np.real(c_h * c.conjugate() / c_s)

        if info['gauss']:
            # The method used below only works for the Gaussian window, because
            # the time derivative and the time multiplicative of the Gaussian
            # are identical.
            tgrad = np.imag(c_h * c.conjugate() / c_s) / info['tfr']

        else:
            # The code below works for any window, and not just the Gaussian

            dg = pderiv(g, difforder=float('inf')) / (2*np.pi)
            c_d = comp_sepdgt(f, dg, a, M, 0)

            # NOTE: There is a bug here in the original octave file as it
            # contains a reshape that uses an undefined variable N.
            # You can get the error with LTFAT 2.1.0 in octave by running for
            # example:
            # gabphasegrad('dgt', rand(16,1), rand(16,1), 4, 16)
            #
            # So we just comment out the corresponding line here, as it appears
            # to be unneeded:
            # c_d.shape = (M, N, W)

            # Compute the instantaneous frequency
            tgrad = -np.imag(c_d * c.conjugate() / c_s)

        return (tgrad, fgrad, c)

    elif method == 'phase':

        # ---------------------------  phase method ------------------------

        (cphase, a) = args

        if not np.isrealobj(cphase):
            raise TypeError("Input phase must be real valued. Use the 'angle'"
                            " function to compute the argument of complex "
                            "numbers.")

        # --- linear method ---
        if cphase.ndim == 3:
            M, N, W = cphase.shape
        else:
            M, N = cphase.shape
        L = N*a
        b = L/M

# NOTE: The following code found in the original octave version of the function
# hasn't been translated here to Python as it is not used:
#        if 0
#
#            # This is the classic phase vocoder algorithm by Flanagan.
#
#            tgrad = cphase-circshift(cphase,[0,-1]);
#            tgrad = tgrad- 2*pi*round(tgrad/(2*pi));
#            tgrad = -tgrad/(2*pi)*L;
#
#            # Phase-lock the angles.
#            TimeInd = (0:(N-1))*a;
#            FreqInd = (0:(M-1))/M;
#
#            phl = FreqInd'*TimeInd;
#            cphase = cphase+2*pi.*phl;
#
#            fgrad = cphase-circshift(cphase,[1,0]);
#            fgrad = fgrad- 2*pi*round(fgrad/(2*pi));
#            fgrad = -fgrad/(2*pi)*L;
#
#        end;

        # This is the classic phase vocoder algorithm by Flanagan modified to
        # yield a second order centered difference approximation.

        # Forward approximation
        tgrad_1 = cphase - np.roll(cphase, -1, axis=1)

        # numpy round function doesn't use the same convention than octave for
        # half-integers but the standard Python round function uses the same
        # convention than octave, so we use the Python standard round in the
        # computation below
        octave_round = np.vectorize(round)
        tgrad_1 = tgrad_1 - 2*np.pi*octave_round(tgrad_1/(2*np.pi))
        # Backward approximation
        tgrad_2 = np.roll(cphase, 1, axis=1) - cphase
        tgrad_2 = tgrad_2 - 2*np.pi*octave_round(tgrad_2/(2*np.pi))
        # Average
        tgrad = (tgrad_1+tgrad_2) / 2

        tgrad = -tgrad / (2*np.pi*a) * L

        # Phase-lock the angles.
        TimeInd = np.arange(N) * a
        FreqInd = np.arange(M) / M

        phl = np.dot(FreqInd.reshape((FreqInd.shape[0], 1)),
                     TimeInd.reshape((1, TimeInd.shape[0])))
        # NOTE: in the following lines, the shape of phl is changed so that
        # broadcasting works in the following addition with cphase when cphase
        # has more than two dimensions
        new_shape = np.ones((len(cphase.shape), ), dtype=int)
        new_shape[0] = phl.shape[0]
        new_shape[1] = phl.shape[1]
        phl = phl.reshape(tuple(new_shape))
        cphase = cphase + 2*np.pi*phl

        # Forward approximation
        fgrad_1 = cphase - np.roll(cphase, -1, axis=0)
        fgrad_1 = fgrad_1 - 2*np.pi*octave_round(fgrad_1/(2*np.pi))
        # Backward approximation
        fgrad_2 = np.roll(cphase, 1, axis=0) - cphase
        fgrad_2 = fgrad_2 - 2*np.pi*octave_round(fgrad_2/(2*np.pi))
        # Average
        fgrad = (fgrad_1+fgrad_2)/2

        fgrad = fgrad/(2*np.pi*b)*L

        return (tgrad, fgrad)

    elif method == 'abs':
        # ---------------------------  abs method ------------------------

        (s, g, a) = args

        if 'difforder' in kwargs:
            difforder = kwargs['difforder']
        else:
            difforder = 4

        if not np.all(s >= 0.):
            raise ValueError('First input argument must be positive or zero.')

        if s.ndim == 3:
            M, N, W = s.shape
        else:
            M, N = s.shape

        L = N*a

        g, info = gabwin(g, a, M, L)

        if not info['gauss']:
            raise ValueError('The window must be a Gaussian window (specified '
                             'as a string or as a dictionary).')

        b = L/M
        # We must avoid taking the log of zero.
        # Therefore we add the smallest possible
        # number
        logs = np.log(s + np.finfo(s.dtype).tiny)

        # XXX REMOVE Add a small constant to limit the dynamic range. This
        # should lessen the problem of errors in the differentiation for points
        # close to (but not exactly) zeros points.
        maxmax = np.max(logs)
        tt = -11.
        logs[logs < (maxmax+tt)] = tt

        fgrad = pderiv(logs, 1, difforder)/(2*np.pi)*info['tfr']
        tgrad = pderiv(logs, 0, difforder)/(2*np.pi*info['tfr'])

        return (tgrad, fgrad)

    else:
        raise ValueError("First argument must be the method name, 'dgt', "
                         "'phase' or 'abs'.")



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the psech function

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import unittest
import random
import numpy as np
from numpy import linalg as LA
from math import sqrt
from ltfatpy.fourier.psech import psech


class TestPsech(unittest.TestCase):
    """ Unittest class for Fourier pgauss"""
    def setUp(self):
        self.L = random.randint(10, 100)

    def tearDown(self):
        print('Test done')

    def test_default(self):
        (g_r1, _) = psech(self.L)
        (g_r2, _) = psech(self.L, tfr=1, width=0.0, bw=0.0, c_f=0.0,
                          centering='wp', delay=0.0, norm='2')
        mess = "\nTest default values with L = " + str(self.L) + "\n"
        self.assertTrue(np.array_equal(g_r1, g_r2), mess)

    def test_exceptions(self):
        mess = "\nException TypeError should be raised with declaration "
        mess += "psech(10.2)\n"
        self.assertRaises(TypeError, psech, 10.2, mess)
        mess = "\nException TypeError should be raised with declaration "
        mess += "psech(10, s=(1,1))\n"
        self.assertRaises(TypeError, psech, 10, s=(1, 1))

    def test_invariance_property(self):
        g = psech(self.L)[0]
        gf = np.fft.fft(g)/sqrt(self.L)
        mess = "Test DFT invariance."
        self.assertAlmostEqual(LA.norm(g-gf), 0, 10, mess)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestPsech)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""This module contains samples of a periodized Hermite function

Ported from ltfat_2.1.0/fourier/pherm.m

.. moduleauthor:: Denis Arrivault
"""

import numpy as np

from ltfatpy.comp.comp_hermite import comp_hermite
from ltfatpy.comp.comp_hermite_all import comp_hermite_all
from ltfatpy.sigproc.normalize import normalize


def pherm(L, order, tfr=1, phase='accurate', orthtype='noorth'):
    """PHERM  Periodized Hermite function

    - Usage:
        | ``g, = pherm(L,order)``
        | ``g, = pherm(L,order,tfr)``
        | ``g, D = pherm(...)``

    - Input parameters:

    :param int L: Length of vector.
    :param order: Order of Hermite function.
    :type order: scalar or numpy.ndarray
    :param float tfr: ratio between time and frequency support. 1 by default
    :param str phase: 'accurate' or 'fast' (see below)
    :param str orthtype: 'noorth', 'polar' or 'qr' (see below).

    - Output parameters:

    :returns: ``(g, D)``
    :rtype: tuple
    :var numpy.ndarray g: The periodized Hermite function
    :var numpy.ndarray D: The eigenvalues of the Discrete
        Fourier Transform corresponding to the Hermite functions.

    ``pherm(L,order,tfr)`` computes samples of a periodized Hermite function
    of order **order**. **order** is counted from 0, so the zero'th order
    Hermite function is the Gaussian.

    The parameter **tfr** determines the ratio between the effective support
    of **g** and the effective support of the DFT of **g**. If :math:`tfr>1`
    then **g** has a wider support than the DFT of **g**.

    ``pherm(L,order)`` does the same setting :math:`tfr=1`.

    If **order** is a vector, ``pherm`` will return a matrix, where each column
    is a Hermite function with the corresponding **order**.

    ``g, D = pherm(...)`` also returns the eigenvalues **D** of the Discrete
    Fourier Transform corresponding to the Hermite functions.

    The returned functions are eigenvectors of the DFT. The Hermite
    functions are orthogonal to all other Hermite functions with a
    different eigenvalue, but eigenvectors with the same eigenvalue are
    not orthogonal (but see the flags below).

    **phase** can take the following values:

      'accurate'  By default it uses a numerically very accurate that
                  computes each Hermite function individually. This is the
                  default

      'fast'      Use a less accurate algorithm that calculates all the
                  Hermite up to a given order at once.

    **orthtype** can take the following values:

      'noorth'    orthonormalization of the Hermite functions. This is the
                  default.

      'polar'     Orthonormalization of the Hermite functions using the
                  polar decomposition orthonormalization method.

      'qr'        Orthonormalization of the Hermite functions using the
                  Gram-Schmidt orthonormalization method (usign ``qr``).

    If you just need to compute a single Hermite function, there is no
    speed difference between the **accurate** and **fast** algorithm.

    - Examples:

    The following plot shows the spectrograms of 4 Hermite functions of
    length 200 with order 1, 10, 100, and 190:::

        >>> import numpy as np
        >>> import matplotlib.pyplot as plt
        >>> from ltfatpy import sgram
        >>> plt.close('all')
        >>> _ = plt.figure()
        >>> _ = plt.subplot(221)
        >>> _ = sgram(pherm(200, 1)[0], nf=True, tc=True, normalization='lin',
        ... colorbar=False)
        >>> _ = plt.subplot(2,2,2)
        >>> _ = sgram(pherm(200, 10)[0], nf=True, tc=True, normalization='lin',
        ... colorbar=False)
        >>> _ = plt.subplot(2,2,3)
        >>> _ = sgram(pherm(200, 100)[0], nf=True, tc=True,
        ... normalization='lin', colorbar=False)
        >>> _ = plt.subplot(2,2,4)
        >>> _ = sgram(pherm(200, 190)[0], nf=True, tc=True,
        ... normalization='lin', colorbar=False)
        >>> plt.show()

    .. image:: images/pherm.png
       :width: 700px
       :alt: spectrograms
       :align: center
    .. seealso:: :func:`~ltfatpy.fourier.pgauss.pgauss`,
                :func:`~ltfatpy.fourier.psech.psech`
    """
    if not np.isscalar(L) or isinstance(L, str):
        raise TypeError("L must be a scalar")

    if not isinstance(L, int):
        raise TypeError('L must be an integer')

    # Parse tfr and order.
    if (not np.isscalar(tfr)):
        raise TypeError('tfr must be a scalar or vector')

    if np.isscalar(order) and not isinstance(order, str):
        W = 1
        order = np.array([order])
    elif isinstance(order, np.ndarray):
        order = order.reshape(-1).copy()
        W = order.shape[0]
    else:
        raise TypeError('order must be a scalar or vector')

    # Calculate W windows.
    if 'accurate' in phase:
        # Calculate W windows.
        g = np.zeros((L, W))
        for w in range(W):
            thisorder = order[w]
            safe = get_safe(thisorder)
            # Outside the interval [-safe,safe]
            # then H(thisorder) is numerically zero.
            nk = int(np.ceil(safe/np.sqrt(L/np.sqrt(tfr))))
            sqrtl = np.sqrt(L)
            lr = np.arange(L)
            for k in range(-nk, nk+1):
                xval = (lr/sqrtl - k*sqrtl) / np.sqrt(tfr)
                g[:, w] = g[:, w] + comp_hermite(thisorder,
                                                 np.sqrt(2*np.pi)*xval)
    else:
        highestorder = np.max(order)
        safe = get_safe(highestorder)
        # Outside the interval [-safe,safe]
        # then H(thisorder) is numerically zero.
        nk = int(np.ceil(safe/np.sqrt(L/np.sqrt(tfr))))
        g = np.zeros((L, highestorder+1))
        sqrtl = np.sqrt(L)
        lr = np.arange(L)
        for k in range(-nk, nk+1):
            xval = (lr/sqrtl - k*sqrtl)/np.sqrt(tfr)
            g = g + comp_hermite_all(highestorder+1, np.sqrt(2*np.pi)*xval)
        g = g[:, order]

    if 'polar' in orthtype:
        # Orthonormalize within each of the 4 eigenspaces
        for ii in range(4):
            subidx = ((order % 4) == ii)
            gsub = g[:, subidx]
            if gsub.size:
                U, _, V = np.linalg.svd(gsub, full_matrices=False)
                gsub = np.dot(U, V)
            else:
                gsub = np.asarray([])
            g[:, subidx] = gsub

    if 'qr' in orthtype:
        # Orthonormalize within each of the 4 eigenspaces
        for ii in range(4):
            subidx = ((order % 4) == ii)
            gsub = g[:, subidx]
            if gsub.size:
                Q, _ = np.linalg.qr(gsub, mode='reduced')
            else:
                Q = np.asarray([])
            g[:, subidx] = Q

    if 'noorth' in orthtype:
        # Just normalize it, no orthonormalization
        g = normalize(g)[0]

    # set up the eigenvalues
    D = np.exp(-1j*order*np.pi/2)
    if W == 1:
        g = g.squeeze()
    return(g, D)


def get_safe(order):
    # These numbers have been computed numerically.
    if order <= 6:
        safe = 4
    else:
        if order <= 18:
            safe = 5
        else:
            if order <= 31:
                safe = 6
            else:
                if order <= 46:
                    safe = 7
                else:
                    # Anything else, use a high number.
                    safe = 12
    return safe

if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of normalized discrete Fourier transform

Ported from ltfat_2.1.0/fourier/dft.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post


def dft(f, N=None, dim=0):
    """Normalized Discrete Fourier Transform

    - Usage:

        | ``c = dft(f)``
        | ``c = dft(f, N, dim)``

    - Input parameters:

    :param numpy.ndarray f: Input array
    :param int N: DFT length
    :param int dim: Axis over which to compute the DFT. By default the first
        axis is used.

    - Output parameters:

    :returns: Normalized discrete Fourier coefficients of **f**
    :rtype: numpy.ndarray

    :func:`~ltfatpy.fourier.dft.dft` computes a normalized or unitary discrete
    Fourier transform. The unitary discrete Fourier transform is computed by

    ..                       L-1
        c(k+1) = 1/sqrt(L) * sum f(l+1)*exp(-2*pi*i*k*l/L)
                             l=0

    .. math::

        c\\left(k+1\\right)=\\frac{1}{\\sqrt{L}}
        \\sum_{l=0}^{L-1}f\\left(l+1\\right)e^{-2\\pi ikl/L}


    for :math:`k=0,\\ldots, L-1`.

    The output of :func:`~ltfatpy.fourier.dft.dft` is a scaled version of the
    output from :func:`numpy.fft.fft`. The function takes the same first three
    arguments as :func:`numpy.fft.fft`. See the help on :func:`numpy.fft.fft`
    for a thorough description.

    .. seealso:: :func:`~ltfatpy.fourier.idft.idft`

    """

    f, N, Ls, W, dim, permutedsize, order = assert_sigreshape_pre(f, N, dim)

    # Force fft along dimension 0, since we have permuted the dimensions
    # manually
    c = np.fft.fft(f, N, 0) / np.sqrt(N)

    c = assert_sigreshape_post(c, dim, permutedsize, order)

    return c



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of gabpars_from_windowsignal calculation

Ported from ltfat_2.1.0/comp/gabpars_from_windowsignal.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

from ltfatpy.comp.comp_sigreshape_pre import comp_sigreshape_pre
from ltfatpy.gabor.dgtlength import dgtlength
from ltfatpy.gabor.gabwin import gabwin
from ltfatpy.tools.postpad import postpad


def gabpars_from_windowsignal(f, g, a, M, L=None):
    """Compute g and L from window and signal

    - Usage:

        | ``(g, info, L) = gabpars_from_windowsignal(f, g, a, M, L)``

    Use this function if you know an input signal and a window
    for the DGT. The function will calculate a transform length L and
    evaluate the window g into numerical form. The signal will be padded and
    returned as a column vector.

    If the transform length is unknown (as it usually is unless explicitly
    specified by the user), set L to be None in the input to this function.
    """
    # Verify f and determine its length
    # Change f to correct shape.
    (f, Ls, W, unused_wasrow, unused_remembershape) = comp_sigreshape_pre(f, 0)

    if L is None:
        # Verify a, M and get L from the signal length f
        L = dgtlength(Ls, a, M)
    else:
        # Verify a, M and get L
        Luser = dgtlength(L, a, M)
        if Luser != L:
            raise ValueError('Incorrect transform length L={0:d} specified.' +
                             'Next valid length is L={1:d}.'.format(L, Luser))

    # Determine the window
    (g, info) = gabwin(g, a, M, L)

    if L < info['gl']:
        raise ValueError('Window is too long.')

    # final cleanup
    f = postpad(f, L)

    # If the signal is single precision, make the window single precision as
    # well to avoid mismatches.
    # if isa(f,'single')
    # g=single(g);
    # end;
    return (f, g, L, Ls, W, info)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the tfplot function

NOTE: The validity of the plotting features of tfplot are not tested here, only
the fact that they can run without error is tested.

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import unittest
import numpy as np
from numpy.testing import assert_array_equal
import matplotlib.pyplot as plt

from ltfatpy.gabor.tfplot import tfplot

# NOTE: The reference values used in the tests correspond to results
# obtained with Octave using ltfat 2.1.0


class TestTfplot(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print('\nStart TestTfplot')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_exceptions(self):
        """Check that the right exceptions are raised when expected
        """
        # coef must be a 2D numpy.ndarray, check that we get the right
        # exceptions if not
        self.assertRaises(TypeError, tfplot, 'test', 1, np.array([0., 1.]))
        self.assertRaises(TypeError, tfplot, 1, 1, np.array([0., 1.]))
        self.assertRaises(TypeError, tfplot, 1., 1, np.array([0., 1.]))
        self.assertRaises(TypeError, tfplot, [0, 2], 1, np.array([0., 1.]))
        self.assertRaises(ValueError, tfplot, np.ones((4,)), 1,
                          np.array([0., 1.]))
        self.assertRaises(ValueError, tfplot, np.ones((4, 3, 2)), 1,
                          np.array([0., 1.]))

    def test_normalization(self):
        """Check that the parameter normalization is used as expected
        """
        normalizations = ('db', 'dbsq', 'linsq', 'linabs')
        zeros_array = np.zeros((2, 2), dtype=np.float64)
        outs = (40. + zeros_array, 20. + zeros_array, 10000. + zeros_array,
                100. + zeros_array)

        inputs = {}
        inputs['coef'] = 0.0+100.0j + zeros_array
        inputs['step'] = 1
        inputs['yr'] = np.array([0., 1.])
        inputs['display'] = False

        for normalization, out_ref in zip(normalizations, outs):
            inputs['normalization'] = normalization
            out = tfplot(**inputs)
            msg = ('tfplot is misusing normalization with inputs ' +
                   str(inputs))
            assert_array_equal(out_ref, out, msg)

        inputs['normalization'] = 'lin'
        self.assertRaises(ValueError, tfplot, **inputs)

        inputs['coef'] = 100. + zeros_array
        out = tfplot(**inputs)
        msg = ('tfplot is misusing normalization with inputs ' + str(inputs))
        assert_array_equal(inputs['coef'], out, msg)

    def test_tc(self):
        """Check that the parameter tc is used as expected
        """
        coefs = (np.array([[1., 2., 3.], [4., 5., 6.]]),
                 np.array([[1., 2., 3., 4.], [5., 6., 7., 8.]]))
        outs = (np.array([[3., 1., 2.], [6., 4., 5.]]),
                np.array([[3., 4., 1., 2.], [7., 8., 5., 6.]]))

        inputs = {}
        inputs['step'] = 1
        inputs['yr'] = np.array([0., 1.])
        inputs['display'] = False
        inputs['normalization'] = 'lin'
        inputs['tc'] = True

        for coef, out_ref in zip(coefs, outs):
            inputs['coef'] = coef
            out = tfplot(**inputs)
            msg = ('tfplot is misusing tc with inputs ' + str(inputs))
            assert_array_equal(out_ref, out, msg)

    def test_clim_dynrange(self):
        """Check that the parameters clim and dynrange are used as expected
        """
        inputs = {}
        inputs['coef'] = np.array([[1., 10., 100.], [1000., 10000., 100000.]])
        inputs['step'] = 1
        inputs['yr'] = np.array([0., 1.])
        inputs['display'] = False

        inputs['clim'] = [20., 80.]
        out_clim = np.array([[20., 20., 40.], [60., 80., 80.]])
        out = tfplot(**inputs)
        msg = ('tfplot is misusing clim with inputs ' + str(inputs))
        assert_array_equal(out_clim, out, msg)

        inputs.pop('clim')
        inputs['dynrange'] = 60.
        out_dynrange = np.array([[40., 40., 40.], [60., 80., 100.]])
        out = tfplot(**inputs)
        msg = ('tfplot is misusing dynrange with inputs ' + str(inputs))
        assert_array_equal(out_dynrange, out, msg)

        inputs['clim'] = [20., 80.]
        out = tfplot(**inputs)
        msg = ('tfplot is misusing clim and dynrange, clim should takes '
               'precedence over dynrange  with inputs ' + str(inputs))
        assert_array_equal(out_clim, out, msg)

    def test_plot(self):
        """Check that the all the plotting sections of tfplot can be run
        """
        # NOTE: To avoid an error when running the tests in a environnement
        # with no $DISPLAY variable defined, we switch matplotlib to a
        # non-interactive backend
        plt.switch_backend("ps")

        coef = np.random.random((5, 4))
        step = 1
        yr = np.array([0., 1.])
        plottypes = ('image', 'contour', 'surf', 'pcolor')
        for plottype in plottypes:
            tfplot(coef, step, yr, plottype=plottype)

        fs = 10.
        tfplot(coef, step, yr, plottype='image', fs=fs)

        clim = (0., 1.)
        tfplot(coef, step, yr, plottype='image', clim=clim)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestTfplot)
    unittest.TextTestRunner(verbosity=2).run(suite)
